from collections import namedtuple

import numpy as np


class MetricsData:
    _Metrics = namedtuple("Metrics", ("horizontal", "vertical"))
    _Values = namedtuple("Values", ("center", "rms", "peak"))
    _Units = namedtuple("Units", ("center", "rms", "peak"))

    def __init__(self):
        self._metrics_values = self._Metrics(self._Values([], [], []), self._Values([], [], []))
        self._metrics_units = self._Metrics(self._Units("px", "px", "px"), self._Units("px", "px", "px"))
        self._scanner_metrics = None
        self._scanner_metrics_values = None
        self._scanner_metrics_errors = None
        self._alignment_position_metrics = None
        self._alignment_position_metrics_values = None
        self._alignment_position_metrics_errors = None

    @property
    def metrics_values(self):
        return self._metrics_values

    @property
    def metrics_units(self):
        return self._metrics_units

    @property
    def scanner_metrics_values(self):
        return self._scanner_metrics_values

    @property
    def scanner_metrics_errors(self):
        return self._scanner_metrics_errors

    @property
    def alignment_position_metrics_values(self):
        return self._alignment_position_metrics_values

    @property
    def alignment_position_metrics_errors(self):
        return self._alignment_position_metrics_errors

    def initialize_scanner_metrics(self):
        self._scanner_metrics = self._Metrics(self._Values([], [], []), self._Values([], [], []))
        self._scanner_metrics_values = self._Metrics(self._Values([], [], []), self._Values([], [], []))
        self._scanner_metrics_errors = self._Metrics(self._Values([], [], []), self._Values([], [], []))

    def initialize_alignment_metrics(self):
        self._alignment_position_metrics = []
        self._alignment_position_metrics_values = []
        self._alignment_position_metrics_errors = []

    def update(self, metrics_horizontal, metrics_vertical):
        self._metrics_values.horizontal.center.append(metrics_horizontal.center)
        self._metrics_values.horizontal.rms.append(metrics_horizontal.rms)
        self._metrics_values.horizontal.peak.append(metrics_horizontal.peak)
        self._metrics_values.vertical.center.append(metrics_vertical.center)
        self._metrics_values.vertical.rms.append(metrics_vertical.rms)
        self._metrics_values.vertical.peak.append(metrics_vertical.peak)

    def reset(self):
        self._metrics_values = self._Metrics(self._Values([], [], []), self._Values([], [], []))

    def update_scanner_metrics(self, append_scanner_metrics, update_scanner_metrics, steps):
        if append_scanner_metrics:
            self._scanner_metrics.horizontal.center.append(self._metrics_values.horizontal.center[-1])
            self._scanner_metrics.horizontal.rms.append(self._metrics_values.horizontal.rms[-1])
            self._scanner_metrics.horizontal.peak.append(self._metrics_values.horizontal.peak[-1])
            self._scanner_metrics.vertical.center.append(self._metrics_values.vertical.center[-1])
            self._scanner_metrics.vertical.rms.append(self._metrics_values.vertical.rms[-1])
            self._scanner_metrics.vertical.peak.append(self._metrics_values.vertical.peak[-1])
        if update_scanner_metrics:
            value, error = self._get_metrics_value_and_error(self._scanner_metrics.horizontal.center, steps)
            self._scanner_metrics_values.horizontal.center.append(value)
            self._scanner_metrics_errors.horizontal.center.append(error)
            value, error = self._get_metrics_value_and_error(self._scanner_metrics.horizontal.rms, steps)
            self._scanner_metrics_values.horizontal.rms.append(value)
            self._scanner_metrics_errors.horizontal.rms.append(error)
            value, error = self._get_metrics_value_and_error(self._scanner_metrics.horizontal.peak, steps)
            self._scanner_metrics_values.horizontal.peak.append(value)
            self._scanner_metrics_errors.horizontal.peak.append(error)
            value, error = self._get_metrics_value_and_error(self._scanner_metrics.vertical.center, steps)
            self._scanner_metrics_values.vertical.center.append(value)
            self._scanner_metrics_errors.vertical.center.append(error)
            value, error = self._get_metrics_value_and_error(self._scanner_metrics.vertical.rms, steps)
            self._scanner_metrics_values.vertical.rms.append(value)
            self._scanner_metrics_errors.vertical.rms.append(error)
            value, error = self._get_metrics_value_and_error(self._scanner_metrics.vertical.peak, steps)
            self._scanner_metrics_values.vertical.peak.append(value)
            self._scanner_metrics_errors.vertical.peak.append(error)

    @staticmethod
    def _get_metrics_value_and_error(metrics, steps):
        last_metrics = np.array(np.split(np.array(metrics), steps))[-1]
        return np.nanmean(last_metrics), np.nanstd(last_metrics)

    def update_alignment_position_metrics(self, append_alignment_position_metrics, update_alignment_position_metrics,
                                          steps):
        if append_alignment_position_metrics:
            self._alignment_position_metrics.append(self._metrics_values.horizontal.rms[-1])
        if update_alignment_position_metrics:
            value, error = self._get_metrics_value_and_error(self._alignment_position_metrics, steps)
            self._alignment_position_metrics_values.append(value)
            self._alignment_position_metrics_errors.append(error)
