import numpy as np
from doocspie.util import Colors

from pyqtgraph import GraphicsLayoutWidget, PlotDataItem


class ProfilerPulseSettingsPlotter:
    _DESY_ORANGE = np.array(Colors.DESY.ORANGE) * 255
    _DESY_CYAN = np.array(Colors.DESY.CYAN) * 255
    _DESY_MAGENTA = np.array(Colors.DESY.MAGENTA) * 255

    def __init__(self):
        self._widget = GraphicsLayoutWidget()

        self._adc = PlotDataItem()
        self._adc_plot = self._widget.addPlot()
        self._adc_plot.addItem(self._adc)
        self._threshold = PlotDataItem()
        self._adc_plot.addItem(self._threshold)
        self._positions = PlotDataItem()
        self._adc_plot.addItem(self._positions)
        self._actual_pulse = PlotDataItem()
        self._adc_plot.addItem(self._actual_pulse)

        self._initialize_plots()

    def _initialize_plots(self):
        self._initialize_adc_plot()

    def _initialize_adc_plot(self):
        self._adc_plot.setMenuEnabled(False)
        self._adc_plot.showAxis("top", show=True)
        self._adc_plot.showAxis("right", show=True)
        self._adc_plot.getAxis("top").setStyle(showValues=False, tickLength=0)
        self._adc_plot.getAxis("right").setStyle(showValues=False, tickLength=0)
        self._adc_plot.getAxis("left").setStyle(tickLength=0)
        self._adc_plot.getAxis("bottom").setStyle(tickLength=0)
        self._threshold.setData(pen={"color": self._DESY_CYAN, "width": 3})
        self._positions.setData(symbol="o", pen=None,
                                symbolBrush=tuple(self._DESY_ORANGE), symbolPen=tuple(self._DESY_ORANGE),
                                symbolSize=8)
        self._actual_pulse.setData(symbol="s", pen=None,
                                   symbolBrush=tuple(self._DESY_MAGENTA), symbolPen=tuple(self._DESY_MAGENTA),
                                   symbolSize=12)
        self._adc_plot.setLabel("bottom", "ADC Bin")
        self._adc_plot.setLabel("left", "Intensity (arb. u.)")

    @property
    def widget(self):
        return self._widget

    def set_adc(self, adc):
        self._adc.setData(adc.bins, adc.values)

    def set_threshold(self, threshold):
        self._threshold.setData(threshold.bins, threshold.values)

    def set_positions(self, positions):
        self._positions.setData(positions.bins, positions.values)

    def set_actual_pulse(self, position, value):
        self._actual_pulse.setData([position], [value])
