class ProfilerService:

    def __init__(self, io_service, profiler_positions, transmission_data):
        self._io_service = io_service
        self._profiler_positions = profiler_positions
        self._profiler_position_data = None
        self._transmission_data = transmission_data
        self._event = None
        self._transmission_readout = None
        self._is_new_data_available = None

    @property
    def profiler_positions(self):
        return self._profiler_positions

    @property
    def transmission_data(self):
        return self._transmission_data

    @property
    def is_new_data_available(self):
        return self._is_new_data_available

    def set_data(self, profiler_position_data):
        self._profiler_position_data = profiler_position_data

    def update_transmission_data(self):
        adc_trace_of_pulses = self._io_service.get(self._transmission_data.adc_address).data
        self._transmission_data.update_pulses(adc_trace_of_pulses)

    def update(self):
        self._update_profiler_position_setpoint()
        self._update_and_process_data()

    def _update_profiler_position_setpoint(self):
        if self._profiler_position_data.is_position_to_set:
            self._set_profiler_position()
            self._check_profiler_position()

    def _set_profiler_position(self):
        if self._profiler_position_data.is_ready_for_new_setpoint():
            self._profiler_position_data.set_reference(self._io_service.get(
                self._profiler_position_data.reference_address).data)
            next_scan_point = self._profiler_position_data.get_next_setpoint()
            if next_scan_point == self._io_service.get(self._profiler_position_data.setpoint_address).data:
                self._profiler_position_data.set_ready_to_check()
            else:
                self._io_service.set(self._profiler_position_data.setpoint_address, next_scan_point)

        if not self._profiler_position_data.is_ready_to_check_position:
            if self._profiler_position_data.reference != self._io_service.get(
                    self._profiler_position_data.reference_address).data:
                self._profiler_position_data.set_ready_to_check()
                self._io_service.set(self._profiler_position_data.setpoint_change_address, 1)

    def _check_profiler_position(self):
        if (self._profiler_position_data.is_ready_to_check_position and
                not self._profiler_position_data.is_ready_for_new_data):
            self._profiler_position_data.check_position(
                self._io_service.get(self._profiler_position_data.readback_address).data,
                self._io_service.get(self._profiler_position_data.setpoint_address).data)

    def _update_and_process_data(self):
        self._transmission_readout = self._io_service.get(self._transmission_data.adc_address)
        self._is_new_data_available = False
        if self._transmission_readout.event != self._event:
            self._event = self._transmission_readout.event
            self._update_data()
            self._is_new_data_available = True

    def _update_data(self):
        self._profiler_position_data.update(self._io_service.get(self._profiler_position_data.readback_address).data)
        self._transmission_data.update(self._transmission_readout.data, self._profiler_position_data)
