from .logger_tool_ui import LoggerToolUi


class LoggerTool:

    def __init__(self, image_data, fit_data, metrics_data):
        self._logger_tool_ui = LoggerToolUi()
        self._image_data = image_data
        self._fit_data = fit_data
        self._metrics_data = metrics_data
        self._is_started = False
        self._create_connections()

    def _create_connections(self):
        self._logger_tool_ui.start_button.clicked.connect(self.start_button_clicked)
        self._logger_tool_ui.stop_button.clicked.connect(self.stop_button_clicked)
        self._logger_tool_ui.reset_button.clicked.connect(self._reset_button_clicked)
        self._logger_tool_ui.add_to_connections_for_changes(self._offline_update_metrics)

    @property
    def ui(self):
        return self._logger_tool_ui

    def start_button_clicked(self):
        self._is_started = True
        self._logger_tool_ui.start_button.setText("running ...")
        self._logger_tool_ui.start_button.setEnabled(False)

    def stop_button_clicked(self):
        if self._is_started:
            self._logger_tool_ui.start_button.setText(self._logger_tool_ui.START_LABEL)
            self._logger_tool_ui.start_button.setEnabled(True)
            self._is_started = False

    def _reset_button_clicked(self):
        if self._image_data.is_not_empty():
            self._metrics_data.reset()
            self._offline_update_metrics()

    def update(self):
        self._logger_tool_ui.plot_projections(self._image_data.projections_values, self._image_data.projections_units)
        self._logger_tool_ui.plot_fits(self._fit_data.fits)
        self._logger_tool_ui.plot_metrics(self._metrics_data.metrics_values, self._metrics_data.metrics_units)

    def offline_update(self):
        if self._image_data.is_not_empty():
            self.update()

    def _offline_update_metrics(self):
        if not self._is_started and self._image_data.is_not_empty():
            self._logger_tool_ui.plot_metrics(self._metrics_data.metrics_values, self._metrics_data.metrics_units)

    def get_export_content(self):
        horizontal_parameters_px = "horizontal parameters (px)"
        vertical_parameters_px = "vertical parameters (px)"
        horizontal_px = "horizontal (px)"
        horizontal_intensity_au = "horizontal intensity (arb.u.)"
        vertical_px = "vertical (px)"
        vertical_intensity_au = "vertical intensity (arb.u.)"
        fit_horizontal_px = "fit horizontal (px)"
        fit_horizontal_intensity_au = "fit horizontal intensity (arb.u.)"
        fit_vertical_px = "fit vertical (px)"
        fit_vertical_intensity_au = "fit vertical intensity (arb.u.)"

        content = {horizontal_parameters_px: [], vertical_parameters_px: [],
                   horizontal_px: [], horizontal_intensity_au: [],
                   vertical_px: [], vertical_intensity_au: [],
                   fit_horizontal_px: [], fit_horizontal_intensity_au: [],
                   fit_vertical_px: [], fit_vertical_intensity_au: [],
                   "average samples": [self._image_data.average_samples],
                   "horizontal parameters label": [self._logger_tool_ui.get_selected_horizontal_label()],
                   "vertical parameters label": [self._logger_tool_ui.get_selected_vertical_label()]}

        if self._image_data.is_not_empty():
            content[horizontal_parameters_px] = self._logger_tool_ui.get_selected_horizontal_metrics(self._metrics_data)
            content[vertical_parameters_px] = self._logger_tool_ui.get_selected_vertical_metrics(self._metrics_data)
            content[horizontal_px] = self._image_data.projections_values.horizontal.x
            content[horizontal_intensity_au] = self._image_data.projections_values.horizontal.y
            content[vertical_px] = self._image_data.projections_values.vertical.x
            content[vertical_intensity_au] = self._image_data.projections_values.vertical.y
            if self._logger_tool_ui.is_fit_visible:
                content[fit_horizontal_px] = self._fit_data.fits.horizontal.x
                content[fit_horizontal_intensity_au] = self._fit_data.fits.horizontal.y
                content[fit_vertical_px] = self._fit_data.fits.vertical.x
                content[fit_vertical_intensity_au] = self._fit_data.fits.vertical.y

        return content
