from PyQt5.QtWidgets import QMainWindow, QVBoxLayout, QWidget, QAction, QTabWidget

from doocspie.pyqt import messages


class RamanometerToolUi(QMainWindow):

    def __init__(self, application_name, version, *tool_uis):
        super().__init__()
        self._application_name = application_name
        self._version = version
        self._tool_uis = tool_uis
        self._main_layout = QVBoxLayout()
        self._print_to_e_log_action = QAction("Print")
        self._export_action = QAction("Export")
        self._click_about_action = QAction("About")
        self._tab_widget = QTabWidget()
        self._create_layout()
        self._create_connections()

    def _create_layout(self):
        self._create_main_layout()
        self._create_menu_bar()
        self._create_tools_tab_widget()

    def _create_main_layout(self):
        main_widget = QWidget()
        main_widget.setLayout(self._main_layout)
        self.setCentralWidget(main_widget)

    def _create_menu_bar(self):
        self._create_file_menu()
        self._create_help_menu()

    def _create_file_menu(self):
        file_menu = self.menuBar().addMenu("File")
        file_menu.addAction(self._print_to_e_log_action)
        file_menu.addAction(self._export_action)

    def _create_help_menu(self):
        self.menuBar().addMenu("Help").addAction(self._click_about_action)

    def _create_tools_tab_widget(self):
        for tool_ui in self._tool_uis:
            self._tab_widget.addTab(tool_ui, tool_ui.name)
        self._main_layout.addWidget(self._tab_widget)

    def _create_connections(self):
        self._click_about_action.triggered.connect(
            lambda: messages.show_about(self, self._application_name, self._version))

    @property
    def application_name(self):
        return self._application_name

    @property
    def version(self):
        return self._version

    @property
    def print_to_e_log_action(self):
        return self._print_to_e_log_action

    @property
    def export_action(self):
        return self._export_action

    @property
    def tab_widget(self):
        return self._tab_widget

    def get_current_ui(self):
        return self._tab_widget.currentWidget()

    def show_window(self):
        self.show()
        self.setMinimumSize(self.width(), self.height())
